/*
******************************************************************************

   LibOVL: extension initialization.

   Copyright (C) 2000 Christoph Egger   [Christoph_Egger@t-online.de]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <string.h>

#include <ggi/internal/ovl.h>

#include <ggi/gg.h> /* We use LibGG to manage config files */


/* Static variables 
 */
static int 	 _ovlLibIsUp       = 0;
static void	*_ovlConfigHandle  = NULL;
static char      _ovlconfstub[512] = OVLCONFFILE;
static char     *_ovlconffile      = _ovlconfstub+OVLTAGLEN;

/* Extension ID. Defaulting to -1 should make segfault on abuse more likely...
 * This is exported so that sublibs can use it.
 */
EXPORTVAR ggi_extid _ovlID = -1;


/* Dummy function which returns -1
   We use this to reset the function pointers */
static int dummyfunc(void)
{
	printf("LibOVL: dummyfunc() of LibOVL called\n");

	return -1;
}

/* Reset all function pointers to dummyfunc */
static void clearfuncs(ovlpriv *priv)
{
	priv->alloc = (void *) dummyfunc;
	priv->check = (void *) dummyfunc;
	priv->free = (void *) dummyfunc;

	priv->setimage = (void *) dummyfunc;
	priv->setoption = (void *) dummyfunc;

	priv->move = (void *) dummyfunc;

	priv->show = (void *) dummyfunc;
	priv->hide = (void *) dummyfunc;
	priv->ishidden = (void *) dummyfunc;
}


/* This function is called by LibGGI when the API-list changes */
static int changed(ggi_visual_t vis, int whatchanged)
{
	printf("LibOVL: changed(%p, 0x%x) of LibOVL called\n",
	       vis, whatchanged);

	switch (whatchanged) {
	case GGI_CHG_APILIST: {
		int i;
		const char *fname;
		ggi_dlhandle *lib;
		char api[GGI_MAX_APILEN], args[GGI_MAX_APILEN];

		clearfuncs(LIBGGI_OVLEXT(vis));

		for(i=0; ggiGetAPI(vis, i, api, args) == 0; i++) {
			strcat(api,"-ovl");
			fname = ggMatchConfig(_ovlConfigHandle, api, NULL);
			if (fname == NULL) {
				/* No special implementation for this sublib */
				continue;
			}
			lib = ggiExtensionLoadDL(vis, fname, args, NULL,
						 "GGIdl_");
		}
	}
	break;
	}
                                
	return 0;
}


/* Initialize the extension
 */
int ggiOvlInit(void)
{
	int err;

	_ovlLibIsUp++;
	if (_ovlLibIsUp > 1) return 0; /* Initialize only at first call. */

	err = ggLoadConfig(_ovlconffile, &_ovlConfigHandle);
	if (err != GGI_OK) {
		fprintf(stderr,"LibOVL: couldn't open %s\n",
			_ovlconffile);
		_ovlLibIsUp--;
		return err;
	}

	_ovlID = ggiExtensionRegister("OVL", sizeof(ovlpriv), changed);

	if (_ovlID < 0) {
		fprintf(stderr,
			"LibOVL: failed to register as extension.\n");
		_ovlLibIsUp--;
		ggFreeConfig(_ovlConfigHandle);
		return _ovlID;
	}

	return 0;
}


/* Deinitialize the extension
 */
int ggiOvlExit(void)
{
	int rc;
    
	if (!_ovlLibIsUp) return -1;

	if (_ovlLibIsUp > 1) {
		/* Exit only at last call */
		_ovlLibIsUp--;
		return 0;
	}
	
	rc = ggiExtensionUnregister(_ovlID);
	ggFreeConfig(_ovlConfigHandle);

	_ovlLibIsUp = 0;

	return rc;
}


/* Attach the extension to a visual
 */
int ggiOvlAttach(ggi_visual_t vis)
{
	int rc;

	printf("LibOVL: ovlAttach(%p) called\n", vis);

	rc = ggiExtensionAttach(vis, _ovlID);
	if (rc == 0) {
		/* We are actually creating the primary instance. */
		memset(LIBGGI_OVLEXT(vis), 0, sizeof(ovlpriv));
		LIBGGI_OVLEXT(vis)->priv = NULL;

		/* Now fake an "API change" so the right libs get loaded */
		changed(vis, GGI_CHG_APILIST);
	}

	return rc;
}


/* Detach the extension from a visual
 */
int ggiOvlDetach(ggi_visual_t vis)
{
	printf("LibOVL: ovlDetach(%p) called\n", vis);

	return ggiExtensionDetach(vis, _ovlID);
}

/* This should be included by exactly _one_ file in each extension library. */
#include <ggi/internal/ggilibinit.h>
